import { __decorate } from 'tslib';
import { EventEmitter, ViewChild, Input, Output, Component, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { CommonModule } from '@angular/common';

class CameraUtil {
    static getAvailableCameraInputs() {
        const sub = new Subject();
        if (!navigator.mediaDevices || !navigator.mediaDevices.enumerateDevices) {
            sub.error('.enumerateDevices method  not supported.');
            return sub.asObservable();
        }
        navigator.mediaDevices.enumerateDevices()
            .then((devices) => {
            sub.next(devices.filter((device) => device.kind === 'videoinput'));
        })
            .catch(err => {
            sub.next(err.message || err);
        });
        return sub.asObservable();
    }
}

var NgxCameraComponent_1;
let NgxCameraComponent = NgxCameraComponent_1 = class NgxCameraComponent {
    constructor() {
        /** camera area width */
        this.width = 640;
        /** camera area height */
        this.height = 480;
        /** define camera image type */
        this.imageType = CameraImageType.jpeg;
        /** should start camera in after view init or not */
        this.startCam = true;
        /** define camera image quality */
        this.imageQuality = 0.9;
        /** event emitter for detected devices */
        this.detectedDevices = new EventEmitter();
        /** event emitter for captured video stream */
        this.stream = new EventEmitter();
        /** event emitter for errors occord at diffrent stages */
        this.errorStream = new EventEmitter();
        /** event emitter for errors occord at diffrent stages */
        this.imageCapture = new EventEmitter();
        /** available camera inputs list */
        this.availableCameraInputs = [];
        /** if video os initialized or not */
        this.videoInitialized = false;
    }
    /**
     * If the given Observable emits, clear canvas
     */
    set clear(clear) {
        if (this.clearSubscription) {
            this.clearSubscription.unsubscribe();
        }
        // Subscribe to events from this Observable to take snapshots
        this.clearSubscription = clear.subscribe(() => {
            const _canvas = this.canvas.nativeElement;
            const context2d = _canvas.getContext('2d');
            context2d.clearRect(0, 0, _canvas.width, _canvas.height);
        });
    }
    /**
     * If the given Observable emits, an image will be captured and emitted through 'imageCapture' EventEmitter
     */
    set trigger(trigger) {
        if (this.triggerSubscription) {
            this.triggerSubscription.unsubscribe();
        }
        // Subscribe to events from this Observable to take snapshots
        this.triggerSubscription = trigger.subscribe(() => {
            this.takeSnapshot();
        });
    }
    /**
     * If the given Observable emits, the active webcam will be switched to the one indicated by the emitted value.
     * @param switchCamera Indicates which webcam to switch to
     *   MediaDeviceInfo: activate the webcam with the given  MediaDeviceInfos
     */
    set switchCamera(switchCamera) {
        if (this.cameraSwitcherSubscription) {
            this.cameraSwitcherSubscription.unsubscribe();
        }
        // Subscribe to events from this Observable to switch video device
        this.cameraSwitcherSubscription = switchCamera.subscribe((value) => {
            if (value instanceof MediaDeviceInfo) {
                this.switchToVideoInput(value);
            }
        });
    }
    /**
     * get native html element from video elementeRef
     * @returns native html element from video elementeRef
     */
    get VideoElement() {
        return this.video.nativeElement;
    }
    /**
     * method to return MediaTrackConstraints from device id and baseMediaTrackConstraints
     * @param deviceId device id to merge MediaTrackConstraints with. you can pass null if you just want to get MediaTrackConstraints.
     * @param baseMediaTrackConstraints MediaTrackConstraints to merge with or you can use this getter ot get default MediaTrackConstraints bu passing null.
     * @returns merged MediaTrackConstraints from given device id and baseMediaTrackConstraints
     */
    static getMediaConstraintsForDevice(deviceId, baseMediaTrackConstraints) {
        const contstraints = baseMediaTrackConstraints ? baseMediaTrackConstraints : NgxCameraComponent_1.DEFAULT_VIDEO_OPTIONS;
        if (deviceId) {
            contstraints.deviceId = { exact: deviceId };
        }
        return contstraints;
    }
    ngAfterViewInit() {
        // detect availabe device and start camera if any device is found;
        this.detectAvailableDevices().subscribe(inputs => {
            if (this.startCamera) {
                if (inputs.length > 0) {
                    this.switchToVideoInput(inputs[0]);
                }
            }
        });
    }
    /**
     * allows you to switch to a spicific camera
     * @param info media info of device that you want to switch camera to
     */
    switchToVideoInput(info) {
        this.videoInitialized = false;
        this.stopMediaTracks();
        const videoConstraints = NgxCameraComponent_1.getMediaConstraintsForDevice(info.deviceId, NgxCameraComponent_1.DEFAULT_VIDEO_OPTIONS);
        this.initWebcam(info.deviceId, videoConstraints);
    }
    /**
     * start a camera with particular device id
     * @param deviceId device id to start camera on
     */
    startCamera(deviceId) {
        this.videoInitialized = false;
        this.initWebcam(deviceId, NgxCameraComponent_1.DEFAULT_VIDEO_OPTIONS);
    }
    /**
     * list all the detected media device with kind video input
     * @returns observable of detected media devices after promis is resolved
     */
    detectAvailableDevices() {
        const sub = new Subject();
        CameraUtil.getAvailableCameraInputs().subscribe(inputs => {
            this.detectedDevices.emit(inputs);
            sub.next(inputs);
        });
        return sub.asObservable();
    }
    /**
     *  initinitialize web cam with device id and constraints and assign all the data streams to respective variables.
     * @param deviceId device id to initinitialize web cam on
     * @param userVideoTrackConstraints user video track constraints.
     */
    initWebcam(deviceId, userVideoTrackConstraints) {
        const video = this.VideoElement;
        if (navigator.mediaDevices && navigator.mediaDevices.getUserMedia) {
            const videoConstraints = NgxCameraComponent_1.getMediaConstraintsForDevice(deviceId, userVideoTrackConstraints);
            navigator.mediaDevices.getUserMedia({ video: videoConstraints })
                .then((stream) => {
                console.log(stream);
                this.videoInitialized = true;
                video.srcObject = stream;
                this.mediaStream = stream;
                video.play();
            });
        }
    }
    /**
     * stops all the media tracks
     */
    stopMediaTracks() {
        if (this.mediaStream && this.mediaStream.getTracks) {
            // getTracks() returns all media tracks (video+audio)
            this.mediaStream.getTracks()
                .forEach((track) => track.stop());
        }
    }
    /**
     * Takes a snapshot of the current webcam's view and emits the image as an event
     */
    takeSnapshot() {
        // set canvas size to actual video size
        const _video = this.VideoElement;
        const dimensions = { width: this.width, height: this.height };
        if (_video.videoWidth) {
            dimensions.width = _video.videoWidth;
            dimensions.height = _video.videoHeight;
        }
        const _canvas = this.canvas.nativeElement;
        _canvas.width = dimensions.width;
        _canvas.height = dimensions.height;
        // paint snapshot image to canvas
        const context2d = _canvas.getContext('2d');
        context2d.drawImage(_video, 0, 0);
        // read canvas content as image
        const mimeType = this.imageType ? this.imageType : 'image/png';
        const quality = this.imageQuality ? this.imageQuality : 0.9;
        const dataUrl = _canvas.toDataURL(mimeType, quality);
        // get the ImageData object from the canvas' context.
        let imageData = null;
        if (this.captureImageData) {
            imageData = context2d.getImageData(0, 0, _canvas.width, _canvas.height);
        }
        console.log(dataUrl);
        this.imageCapture.next(new CameraImage(dataUrl, mimeType, imageData));
    }
};
/** default video options */
NgxCameraComponent.DEFAULT_VIDEO_OPTIONS = { aspectRatio: 1, height: 480, width: 640, facingMode: 'user' };
__decorate([
    ViewChild('video', { static: true })
], NgxCameraComponent.prototype, "video", void 0);
__decorate([
    ViewChild('canvas', { static: true })
], NgxCameraComponent.prototype, "canvas", void 0);
__decorate([
    Input()
], NgxCameraComponent.prototype, "width", void 0);
__decorate([
    Input()
], NgxCameraComponent.prototype, "height", void 0);
__decorate([
    Input()
], NgxCameraComponent.prototype, "imageType", void 0);
__decorate([
    Input()
], NgxCameraComponent.prototype, "startCam", void 0);
__decorate([
    Input()
], NgxCameraComponent.prototype, "imageQuality", void 0);
__decorate([
    Output()
], NgxCameraComponent.prototype, "detectedDevices", void 0);
__decorate([
    Output()
], NgxCameraComponent.prototype, "stream", void 0);
__decorate([
    Output()
], NgxCameraComponent.prototype, "errorStream", void 0);
__decorate([
    Output()
], NgxCameraComponent.prototype, "imageCapture", void 0);
__decorate([
    Input()
], NgxCameraComponent.prototype, "clear", null);
__decorate([
    Input()
], NgxCameraComponent.prototype, "trigger", null);
__decorate([
    Input()
], NgxCameraComponent.prototype, "switchCamera", null);
NgxCameraComponent = NgxCameraComponent_1 = __decorate([
    Component({
        selector: 'ngx-camera',
        template: "<div class=\"canvas-wrapper\" [ngStyle]=\"{'width': width+'px','height': height+'px'}\">\r\n  <video #video [width]='width' [height]=\"height\"></video>\r\n  <canvas #canvas></canvas>\r\n</div>\r\n",
        styles: [".canvas-wrapper{position:relative}.canvas-wrapper canvas,.canvas-wrapper video{position:absolute;top:0;left:0}"]
    })
], NgxCameraComponent);
var WebcamMirrorProperties;
(function (WebcamMirrorProperties) {
    WebcamMirrorProperties["auto"] = "auto";
    WebcamMirrorProperties["always"] = "always";
    WebcamMirrorProperties["never"] = "never"; // ["auto", "always", "never"]
})(WebcamMirrorProperties || (WebcamMirrorProperties = {}));
var CameraImageType;
(function (CameraImageType) {
    CameraImageType["jpeg"] = "image/jpeg";
    CameraImageType["png"] = "image/png";
})(CameraImageType || (CameraImageType = {}));
class CameraImage {
    constructor(imageAsDataUrl, mimeType, imageData) {
        this._mimeType = null;
        this._imageAsBase64 = null;
        this._imageAsDataUrl = null;
        this._imageData = null;
        this._mimeType = mimeType;
        this._imageAsDataUrl = imageAsDataUrl;
        this._imageData = imageData;
    }
    /**
     * Extracts the Base64 data out of the given dataUrl.
     * @param dataUrl the given dataUrl
     * @param mimeType the mimeType of the data
     */
    static getDataFromDataUrl(dataUrl, mimeType) {
        return dataUrl.replace(`data:${mimeType};base64,`, '');
    }
    /**
     * Get the base64 encoded image data
     * @returns base64 data of the image
     */
    get imageAsBase64() {
        return this._imageAsBase64 ? this._imageAsBase64
            : this._imageAsBase64 = CameraImage.getDataFromDataUrl(this._imageAsDataUrl, this._mimeType);
    }
    /**
     * Get the encoded image as dataUrl
     * @returns the dataUrl of the image
     */
    get imageAsDataUrl() {
        return this._imageAsDataUrl;
    }
    /**
     * Get the ImageData object associated with the canvas' 2d context.
     * @returns the ImageData of the canvas's 2d context.
     */
    get imageData() {
        return this._imageData;
    }
}

let NgxCameraModule = class NgxCameraModule {
};
NgxCameraModule = __decorate([
    NgModule({
        declarations: [NgxCameraComponent],
        imports: [
            CommonModule
        ],
        exports: [NgxCameraComponent]
    })
], NgxCameraModule);

/*
 * Public API Surface of ngx-camera
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CameraImage, CameraImageType, CameraUtil, NgxCameraComponent, NgxCameraModule };
//# sourceMappingURL=ngx-camera.js.map
